/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.modlogan.org
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: datatype.c,v 1.21 2003/01/06 11:11:44 ostborn Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "config.h"
#include "mdatatypes.h"
#include "datatype.h"

/* begin of WebHist */

int mdata_WebHist_to_xml(gzFile *fd, mdata *data) {
#define WR(x,f) \
	gzprintf(fd, "<%s>%"#f"</%s>\n", #x, data->data.hist->data.web.x, #x);

	WR(hits, ld);
	WR(files, ld);
	WR(pages, ld);
	WR(visits, ld);
	WR(hosts, ld);
	WR(xfersize, .0f);
#undef WR
#define WR(x,f) \
	gzprintf(fd, "<%s>%"#f"</%s>\n", #x, data->data.hist->x, #x);
	WR(year, d);
	WR(month, d);
	WR(week, d);
	WR(days_passed, d);
#undef WR

	return 0;
}

int mdata_WebHist_free(mdata *data) {
	if (!data) return -1;
	if (data->type != M_DATA_TYPE_WEBHIST) return -1;
	
	free(data->data.hist);

	return 0;
}

int mdata_WebHist_setdata(mdata *data, const char *str,
	unsigned int hits, unsigned int files,
	unsigned int pages, unsigned int visits,
	unsigned int hosts, unsigned int year,
	unsigned int month, unsigned int week,
	unsigned int days_passed, double xfersize ) {

	data->key = strdup(str);
	assert(data->key);

	if (data->type == M_DATA_TYPE_UNSET)
		data->type		= M_DATA_TYPE_WEBHIST;

	data->data.hist->data.web.hits		= hits;
	data->data.hist->data.web.files	= files;
	data->data.hist->data.web.pages	= pages;
	data->data.hist->data.web.visits	= visits;
	data->data.hist->data.web.hosts	= hosts;

	data->data.hist->year		= year;
	data->data.hist->month	= month;
	data->data.hist->week		= week;

	data->data.hist->days_passed	= days_passed;
	data->data.hist->data.web.xfersize	= xfersize;

	data->data.hist->type = M_HIST_TYPE_WEB;

	return 0;
}

int mdata_WebHist_from_xml(void *user_data, int tagtype, const xmlChar *value, const xmlChar **attrs) {
	int i;

	const mdata_values data_values[] = {
		{ "hits", M_DATA_FIELDTYPE_LONG },
		{ "files", M_DATA_FIELDTYPE_LONG },
		{ "pages", M_DATA_FIELDTYPE_LONG },
		{ "visits", M_DATA_FIELDTYPE_LONG },
		{ "hosts", M_DATA_FIELDTYPE_LONG },
		{ "year", M_DATA_FIELDTYPE_LONG },
		{ "month", M_DATA_FIELDTYPE_LONG },
		{ "week", M_DATA_FIELDTYPE_LONG },
		{ "days_passed", M_DATA_FIELDTYPE_LONG },
		{ "xfersize", M_DATA_FIELDTYPE_DOUBLE },

		{ NULL, M_DATA_FIELDTYPE_UNSET }
	};


	mstate_stack *m = user_data;

//	M_WP();

	switch(tagtype) {
	case M_TAG_BEGIN:
		for (i = 0; data_values[i].string != NULL && 0 != strcmp(data_values[i].string, value); i++)
			;

		if (data_values[i].string == NULL) {
			M_DEBUG1(M_DEBUG_LEVEL_ERRORS, M_DEBUG_SECTION_INIT, M_DEBUG_LEVEL_ERRORS,
				 "unknown tag '%s'\n",
				 value);
			return -1;
		}

#define SET_DATA(x) \
	m->st[m->st_depth].data = &(((mdata *)(m->st[m->st_depth-1].data))->x);

		switch(i) {
		case 0: SET_DATA(data.hist->data.web.hits);  break;
		case 1: SET_DATA(data.hist->data.web.files); break;
		case 2: SET_DATA(data.hist->data.web.pages); break;
		case 3: SET_DATA(data.hist->data.web.visits); break;
		case 4: SET_DATA(data.hist->data.web.hosts); break;
		case 5: SET_DATA(data.hist->year); break;
		case 6: SET_DATA(data.hist->month); break;
		case 7: SET_DATA(data.hist->week); break;
		case 8: SET_DATA(data.hist->days_passed); break;
		case 9: SET_DATA(data.hist->data.web.xfersize); break;
		default:
			return -1;
		}
#undef SET_DATA
		/* the same for all 4 values */
		m->st[m->st_depth].function = mdata_insert_value;
		m->st[m->st_depth].type = data_values[i].type;

		break;
	case M_TAG_END:
		((mdata *)(m->st[m->st_depth-1].data))->type = M_DATA_TYPE_WEBHIST;
		
		((mdata *)(m->st[m->st_depth-1].data))->data.hist->type = M_HIST_TYPE_WEB;

		switch(m->st[m->st_depth-2].type) {
		case M_DATA_FIELDTYPE_LIST: {
			mlist *h = m->st[m->st_depth-2].data;
			mdata *data = m->st[m->st_depth-1].data;

			mlist_insert_sorted(h, data);
			break;
		}
		default:
			M_WP();
		}

		break;
	case M_TAG_TEXT:
		M_WP();
		break;
	default:
		M_DEBUG1(M_DEBUG_LEVEL_ERRORS, M_DEBUG_SECTION_INIT, M_DEBUG_LEVEL_ERRORS,
			 "can't handle tagtype '%d'\n",
			 tagtype);
		return -1;
	}

	return 0;
}

int mdata_WebHist_append(mdata *dst, mdata *src) {

#define IS_EQ(x) (dst->data.hist->x == src->data.hist->x)
#define SET(x) dst->data.hist->data.web.x += src->data.hist->data.web.x
	if (IS_EQ(year) && IS_EQ(month) /* && IS_EQ(days_passed) && IS_EQ(week)*/) {
		fprintf(stderr, "%s.%d: combining hist.data.web. i hope this is intended here.\n", __FILE__, __LINE__ );
		SET(hits);
		SET(files);
		SET(pages);
		SET(visits);
		SET(hosts);

		SET(xfersize);
	} else {
		fprintf(stderr, "%s.%d: combining hist.data.web. -- not done.\n", __FILE__, __LINE__ );
	}
#undef SET
#undef IS_EQ
	return 0;
}

mdata *mdata_WebHist_init() {
	mdata *data = mdata_init();
	assert(data);

	data->key		= NULL;
	data->type		= M_DATA_TYPE_WEBHIST;
	
	data->data.hist         = malloc(sizeof(data_History));
	memset(data->data.hist, 0, sizeof(data_History));

#ifdef DEBUG_DATATYPES
	fprintf(stderr, "%s.%d: got %ld (%d)\n", __FILE__, __LINE__, data->id, data->type);
#endif

/* init specifics */
	data->data.hist->data.web.hits		= 0;
	data->data.hist->data.web.files	= 0;
	data->data.hist->data.web.pages	= 0;
	data->data.hist->data.web.visits	= 0;
	data->data.hist->data.web.hosts	= 0;

	data->data.hist->year		= 0;
	data->data.hist->month	= 0;
	data->data.hist->week		= 0;

	data->data.hist->days_passed	= 0;
	data->data.hist->data.web.xfersize	= 0;

	data->data.hist->type = M_HIST_TYPE_WEB;

	return data;
}

mdata *mdata_WebHist_copy(mdata *src) {
	mdata *data = mdata_WebHist_init();
	assert(data);

	mdata_WebHist_setdata(data, src->key,
		src->data.hist->data.web.hits, src->data.hist->data.web.files,
		src->data.hist->data.web.pages, src->data.hist->data.web.visits,
		src->data.hist->data.web.hosts, src->data.hist->year,
		src->data.hist->month, src->data.hist->week,
		src->data.hist->days_passed, src->data.hist->data.web.xfersize
		);

	return data;
}

mdata *mdata_WebHist_create(const char *str,
	unsigned int hits, unsigned int files,
	unsigned int pages, unsigned int visits,
	unsigned int hosts, unsigned int year,
	unsigned int month, unsigned int week,
	unsigned int days_passed, double xfersize ) {

	mdata *data = mdata_WebHist_init();
	assert(data);
	mdata_WebHist_setdata(data, str,
		hits, files,
		pages, visits,
		hosts, year,
		month, week,
		days_passed, xfersize
	);

	return data;
}

mdata  *mdata_WebHist_create_by_state(mstate *state) {
	char buf[7];
	long i, pages, hits, files, visits, hosts, last_day = 0;
	double xfersize;
	struct tm tm;
	mstate_web *staweb;
	time_t timestamp;

	mdata *data = mdata_WebHist_init();
	assert(data);

	if (state == NULL) return NULL;

	timestamp = state->timestamp;

	tm = *localtime(&(state->timestamp));

	sprintf(buf, "%04d%02d", state->year, state->month);

	pages	= 0;
	hits	= 0;
	files	= 0;
	visits	= 0;
	hosts	= 0;
	xfersize = 0;

	if (state->ext && state->ext_type == M_STATE_TYPE_WEB) {
		staweb = state->ext;

		for ( i = 0; i < 31; i++) {
			files		+= staweb->days[i].files;
			xfersize	+= staweb->days[i].xfersize;
			hits		+= staweb->days[i].hits;
			hosts		+= staweb->days[i].hosts;
			pages		+= staweb->days[i].pages;
			visits		+= staweb->days[i].visits;
			if (staweb->days[i].hits) last_day = i+1;
		}
	}
	mdata_WebHist_setdata(data, buf,
		hits, files,
		pages, visits,
		hosts, state->year,
		state->month, 0,
		last_day, xfersize
	);

	return data;
}

/* end of WebHist */
