/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.modlogan.org
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: parse.c,v 1.5 2003/04/18 18:40:41 ostborn Exp $
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>

#include "mlocale.h"
#include "mplugins.h"
#include "mrecord.h"
#include "mdatatypes.h"
#include "misc.h"

#include "plugin_config.h"
#if 0
#define DEBUG_PCRE
#endif

int parse_timestamp(mconfig *ext_conf, const char *str, mlogrec *record) {
#define N 20 + 1
	int ovector[3 * N], n;
	char buf[10];
	struct tm tm;
	config_input *conf = ext_conf->plugin_conf;

	if ((n = pcre_exec(conf->match_timestamp, NULL, str, strlen(str), 0, 0, ovector, 3 * N)) < 0) {
		if (n == PCRE_ERROR_NOMATCH) {
			M_DEBUG1(ext_conf->debug_level, M_DEBUG_SECTION_PARSING, M_DEBUG_LEVEL_ERRORS,
				 "string doesn't match: %s\n", str);
			return M_RECORD_CORRUPT;
		} else {
			M_DEBUG1(ext_conf->debug_level, M_DEBUG_SECTION_PARSING, M_DEBUG_LEVEL_ERRORS,
				 "execution error while matching: %d\n", n);
			return M_RECORD_HARD_ERROR;
		}
	}

	memset(&tm, 0, sizeof(struct tm));

	/* everything has matched, take the different pieces and be happy :) */
	pcre_copy_substring(str, ovector, n, 1, buf, sizeof(buf));
	tm.tm_mday = strtol(buf, NULL, 10);

	pcre_copy_substring(str, ovector, n, 2, buf, sizeof(buf));
	tm.tm_mon = strtol(buf, NULL, 10)-1;

	pcre_copy_substring(str, ovector, n, 3, buf, sizeof(buf));
	tm.tm_year = strtol(buf, NULL, 10)-1900;

	pcre_copy_substring(str, ovector, n, 4, buf, sizeof(buf));
	tm.tm_hour = strtol(buf, NULL, 10);
	pcre_copy_substring(str, ovector, n, 5, buf, sizeof(buf));
	tm.tm_min = strtol(buf, NULL, 10);
	pcre_copy_substring(str, ovector, n, 6, buf, sizeof(buf));
	tm.tm_sec = strtol(buf, NULL, 10);

	record->timestamp = mktime (&tm);

	return M_RECORD_NO_ERROR;
#undef  N
}

int parse_record_pcre(mconfig *ext_conf, mlogrec *record, buffer *b) {
#define N 20 + 1
	const char **list;
	int ovector[3 * N], n, ret;
#ifdef DEBUG_PCRE
	int i;
#endif

	config_input *conf = ext_conf->plugin_conf;
	mlogrec_mail *recmail = NULL;
	mlogrec_mail_virus *recvir = NULL;

	record->ext_type = M_RECORD_TYPE_MAIL;
	record->ext = mrecord_init_mail();

	recmail = record->ext;

	if (recmail == NULL) return -1;

	recmail->ext = mrecord_init_mail_virus();
	recmail->ext_type = M_RECORD_TYPE_MAIL_VIRUS;

	recvir = recmail->ext;

	if (recvir == NULL) return -1;

	if ((n = pcre_exec(conf->match_qmailscanner, conf->match_qmailscanner_extra, b->ptr, b->used - 1, 0, 0, ovector, 3 * N)) < 0) {
		if (n == PCRE_ERROR_NOMATCH) {
			fprintf(stderr, "%s.%d: string doesn't match: %s\n", __FILE__, __LINE__, b->ptr);
		} else {
			fprintf(stderr, "%s.%d: execution error while matching: %d\n", __FILE__, __LINE__, n);
		}
		return -1;
	}

	if (n) {
		pcre_get_substring_list(b->ptr, ovector, n, &list);

/* the meaning of the different fields (qmail-scanner) */
	/*
	   1 - timestamp
	   2 - sender
	   3 - receipient
	   4 - subject
	   5 - name of virus
	   6 - scanner
	*/
		switch ((ret = parse_timestamp(ext_conf, list[1], record))) {
		case M_RECORD_NO_ERROR:
			break;
		case M_RECORD_HARD_ERROR:
			fprintf(stderr, "%s.%d: parse_timestamp died on %s\n", __FILE__, __LINE__, b->ptr);
			free(list);
			return M_RECORD_HARD_ERROR;
		case M_RECORD_IGNORED:
			free(list);
			return M_RECORD_IGNORED;
		case M_RECORD_CORRUPT:
			free(list);
			return M_RECORD_CORRUPT;
		default:
			fprintf(stderr, "%s.%d: parse_timestamp return a unknown ret-value on %d\n",
				__FILE__, __LINE__,
				ret);
			free(list);
			return M_RECORD_HARD_ERROR;
		}

		recmail->sender = malloc(strlen(list[2]) + 1);
		strcpy(recmail->sender, list[2]);

		recmail->receipient = malloc(strlen(list[3]) + 1);
		strcpy(recmail->receipient, list[3]);

		recvir->subject = malloc(strlen(list[4]) + 1);
		strcpy(recvir->subject, list[4]);

		recvir->virus = malloc(strlen(list[5]) + 1);
		strcpy(recvir->virus, list[5]);

		recvir->scanner = malloc(strlen(list[6]) + 1);
		strcpy(recvir->scanner, list[6]);
#ifdef DEBUG_PCRE
		for (i = 0; i < n; i++) {
			printf("%d: %s\n", i, list[i]);
		}
#endif
		free(list);
	}

	return 0;
#undef  N
}


int mplugins_input_qmailscanner_get_next_record(mconfig *ext_conf, mlogrec *record) {
	int ret = 0;
	config_input *conf = ext_conf->plugin_conf;

	if (record == NULL) return M_RECORD_HARD_ERROR;

	/* fill the line buffer */
	if (NULL == mgets(&(conf->inputfile), conf->buf)) return M_RECORD_EOF;
	
	ret = parse_record_pcre   (ext_conf, record, conf->buf);
	
	if (ret == M_RECORD_CORRUPT) {
		M_DEBUG1(ext_conf->debug_level, M_DEBUG_SECTION_PARSING, M_DEBUG_LEVEL_WARNINGS,
			 "affected Record: %s\n",
			 conf->buf->ptr
			 );
	}
	return ret;
}
